@@ -1,14 +1,13 @@
 """Base mongoDB data object layers"""
 
 from functools import wraps, cached_property
-from typing import Mapping, Optional, Union, Iterable
-from collections.abc import KeysView, ValuesView, ItemsView
+from typing import Mapping, Optional, Union
 from collections import ChainMap
+from dol.base import Store
 
 from pymongo import MongoClient
 
-from dol import KvReader
-from dol import Collection as DolCollection
+from dol import KvReader, Collection as DolCollection, BaseValuesView, BaseItemsView
 
 from mongodol.constants import ID, PyMongoCollectionSpec, end_of_cursor, DFLT_TEST_DB
 from mongodol.util import (
@@ -86,40 +85,6 @@ class MongoCollectionCollection(DolCollection):
         )
 
 
-class MongoValuesView(ValuesView):
-    def __contains__(self, v):
-        m = self._mapping
-        cursor = m.mgc.find(filter=m._merge_with_filt(v), projection=())
-        return next(cursor, end_of_cursor) is not end_of_cursor
-
-    def __iter__(self):
-        m = self._mapping
-        return m.mgc.find(filter=m.filter, projection=m._getitem_projection)
-
-    # def distinct(self, key, filter=None, **kwargs):
-    #     m = self._mapping
-    #     # TODO: Check if this is correct (what about $ cases?): filter=m._merge_with_filt(filter)
-    #     return m.mgc.distinct(key, filter=m._merge_with_filt(filter), **kwargs)
-    #
-    # unique = distinct
-
-
-class MongoItemsView(ItemsView):
-    def __contains__(self, item):
-        m = self._mapping
-        k, v = item
-        # TODO: How do we have cursor return no data (here still has _id)
-        cursor = m.mgc.find(filter=dict(v, **m._merge_with_filt(k)), projection=())
-        # return cursor
-        return next(cursor, end_of_cursor) is not end_of_cursor
-
-    def __iter__(self):
-        m = self._mapping
-        for doc in m.mgc.find(filter=m.filter, projection=m._items_projection):
-            key = {k: doc.pop(k) for k in m.key_fields}
-            yield key, doc
-
-
 class MongoCollectionReader(MongoCollectionCollection, KvReader):
     """A base class to read from a mongo collection, or subset thereof, with the Mapping
     (i.e. dict-like) interface.
@@ -159,12 +124,9 @@ class MongoCollectionReader(MongoCollectionCollection, KvReader):
     ``s.keys()``, ``s.values()``, and ``s.items()`` are ``collections.abc.MappingViews`` instances
     (specialized for mongo).
 
-    >>> type(s.keys()).__name__
-    'KeysView'
-    >>> type(s.values()).__name__
-    'MongoValuesView'
-    >>> type(s.items()).__name__
-    'MongoItemsView'
+    >>> assert type(s.keys()) == s.KeysView
+    >>> assert type(s.values()) == s.ValuesView
+    >>> assert type(s.items()) == s.ItemsView
 
     Recall that ``collections.abc.MappingViews`` have many set-like functionalities:
 
@@ -196,19 +158,30 @@ class MongoCollectionReader(MongoCollectionCollection, KvReader):
     """
 
     _projections_are_flattened = False
-    _wrap_for_method = {
-        'values': MongoValuesView,
-        'items': MongoItemsView,
-    }
+
+    class ValuesView(BaseValuesView):
+        def __contains__(self, v):
+            return self._mapping.contains_value(v)
+
+        def __iter__(self):
+            return self._mapping.iter_values()
+
+    class ItemsView(BaseItemsView):
+        def __contains__(self, item):
+            return self._mapping.contains_item(item)
+
+        def __iter__(self):
+            return self._mapping.iter_items()
 
     def __init__(
         self,
         mgc: Union[PyMongoCollectionSpec, KvReader] = None,
         filter: Optional[dict] = None,
-        iter_projection: ProjectionSpec = (ID,),
+        iter_projection: ProjectionSpec = None,
         getitem_projection: ProjectionSpec = None,
         **mgc_find_kwargs,
     ):
+        iter_projection = iter_projection or (ID,)
         if not isinstance(iter_projection, dict):
             iter_projection = {k: True for k in iter_projection}
         assert iter_projection is not None, 'iter_projection cannot be None'
@@ -225,18 +198,38 @@ class MongoCollectionReader(MongoCollectionCollection, KvReader):
             filter=self._merge_with_filt(k), projection=self._getitem_projection,
         )
 
-    # delete once verified that it's unnecessary
-    # def keys(self):
-    #     return KeysView(self)
+    def contains_value(self, v):
+        cursor = self.mgc.find(
+            filter=self._merge_with_filt(v), projection=(), **self._mgc_find_kwargs
+        )
+        return next(cursor, end_of_cursor) is not end_of_cursor
 
-    ValuesView = MongoValuesView
-    ItemsView = MongoItemsView
+    def iter_values(self):
+        return self.mgc.find(
+            filter=self.filter,
+            projection=self._getitem_projection,
+            **self._mgc_find_kwargs,
+        )
 
-    def values(self) -> MongoValuesView:
-        return MongoValuesView(self)
+    def contains_item(self, item):
+        k, v = item
+        # TODO: How do we have cursor return no data (here still has _id)
+        cursor = self.mgc.find(
+            filter=dict(v, **self._merge_with_filt(k)),
+            projection=(),
+            **self._mgc_find_kwargs,
+        )
+        return next(cursor, end_of_cursor) is not end_of_cursor
 
-    def items(self) -> MongoItemsView:
-        return MongoItemsView(self)
+    def iter_items(self):
+        cursor = self.mgc.find(
+            filter=self.filter,
+            projection=self._items_projection,
+            **self._mgc_find_kwargs,
+        )
+        for doc in cursor:
+            key = {k: doc.pop(k) for k in self.key_fields}
+            yield (key, doc)
 
     @cached_property
     def _items_projection(self):
@@ -494,8 +487,8 @@ class MongoCollectionPersister(MongoCollectionReader):
 
 class MongoClientReader(KvReader):
     @wraps(MongoClient.__init__)
-    def __init__(self, **mongo_client_kwargs):
-        self._mongo_client = MongoClient(**mongo_client_kwargs)
+    def __init__(self, *mongo_client_args, **mongo_client_kwargs):
+        self._mongo_client = MongoClient(*mongo_client_args, **mongo_client_kwargs)
 
     def __iter__(self):
         yield from self._mongo_client.list_database_names()
@@ -540,3 +533,27 @@ class MongoDbReader(KvReader):
 
     def __getitem__(self, k):
         return self.collection_store_cls(self.db[k])
+
+
+class MongoBaseStore(Store):
+    def contains_value(self, v):
+        return self.store.contains_value(self._data_of_obj(v))
+
+    def iter_values(self):
+        return map(self._obj_of_data, self.store.iter_values())
+
+    def contains_item(self, item):
+        k, v = item
+        return self.store.contains_item((self._id_of_key(k), self._data_of_obj(v)))
+
+    def iter_items(self):
+        yield from (
+            (self._key_of_id(key), self._obj_of_data(doc))
+            for key, doc in self.store.iter_items()
+        )
+
+    def append(self, v):
+        return self.store.append(self._data_of_obj(v))
+
+    def extend(self, values):
+        return self.store.extend(list(map(self._data_of_obj, values)))
